---
title: redis_list
type: input
status: stable
categories: ["Services"]
---

<!--
     THIS FILE IS AUTOGENERATED!

     To make changes please edit the contents of:
     lib/input/redis_list.go
-->

import Tabs from '@theme/Tabs';
import TabItem from '@theme/TabItem';


Pops messages from the beginning of a Redis list using the BLPop command.


<Tabs defaultValue="common" values={[
  { label: 'Common', value: 'common', },
  { label: 'Advanced', value: 'advanced', },
]}>

<TabItem value="common">

```yaml
# Common config fields, showing default values
input:
  label: ""
  redis_list:
    url: tcp://localhost:6379
    key: benthos_list
```

</TabItem>
<TabItem value="advanced">

```yaml
# All config fields, showing default values
input:
  label: ""
  redis_list:
    url: tcp://localhost:6379
    kind: simple
    master: ""
    tls:
      enabled: false
      skip_cert_verify: false
      enable_renegotiation: false
      root_cas_file: ""
      client_certs: []
    key: benthos_list
    timeout: 5s
```

</TabItem>
</Tabs>

## Fields

### `url`

The URL of the target Redis server. Database is optional and is supplied as the URL path. `tcp` scheme is the same as `redis`


Type: `string`  
Default: `"tcp://localhost:6379"`  

```yaml
# Examples

url: :6397

url: localhost:6397

url: redis://localhost:6379

url: redis://localhost:6379/1

url: redis://localhost:6379/1,redis://localhost:6380/1
```

### `kind`

Specifies a simple, cluster-aware, or failover-aware redis client.


Type: `string`  
Default: `"simple"`  

```yaml
# Examples

kind: simple

kind: cluster

kind: failover
```

### `master`

Name of the redis master when `kind` is `failover`


Type: `string`  
Default: `""`  

```yaml
# Examples

master: mymaster
```

### `tls`

Custom TLS settings can be used to override system defaults.


Type: `object`  

### `tls.enabled`

Whether custom TLS settings are enabled.


Type: `bool`  
Default: `false`  

### `tls.skip_cert_verify`

Whether to skip server side certificate verification.


Type: `bool`  
Default: `false`  

### `tls.enable_renegotiation`

Whether to allow the remote server to repeatedly request renegotiation. Enable this option if you're seeing the error message `local error: tls: no renegotiation`.


Type: `bool`  
Default: `false`  
Requires version 3.45.0 or newer  

### `tls.root_cas_file`

An optional path of a root certificate authority file to use. This is a file, often with a .pem extension, containing a certificate chain from the parent trusted root certificate, to possible intermediate signing certificates, to the host certificate.


Type: `string`  
Default: `""`  

```yaml
# Examples

root_cas_file: ./root_cas.pem
```

### `tls.client_certs`

A list of client certificates to use. For each certificate either the fields `cert` and `key`, or `cert_file` and `key_file` should be specified, but not both.


Type: `array`  
Default: `[]`  

```yaml
# Examples

client_certs:
  - cert: foo
    key: bar

client_certs:
  - cert_file: ./example.pem
    key_file: ./example.key
```

### `tls.client_certs[].cert`

A plain text certificate to use.


Type: `string`  
Default: `""`  

### `tls.client_certs[].key`

A plain text certificate key to use.


Type: `string`  
Default: `""`  

### `tls.client_certs[].cert_file`

The path to a certificate to use.


Type: `string`  
Default: `""`  

### `tls.client_certs[].key_file`

The path of a certificate key to use.


Type: `string`  
Default: `""`  

### `key`

The key of a list to read from.


Type: `string`  
Default: `"benthos_list"`  

### `timeout`

The length of time to poll for new messages before reattempting.


Type: `string`  
Default: `"5s"`  


